Package: alphaN
Title: Set Alpha Based on Sample Size Using Bayes Factors
Version: 0.2.0
Authors@R: c(
    person("Jesper", "Wulff", , "jwulff@econ.au.dk", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-7976-0939")),
    person("Luke", "Taylor", , "lntaylor@econ.au.dk", role = "aut")
  )
Description: Sets the alpha level for coefficients in a regression model
    as a decreasing function of the sample size through the use of
    Jeffreys' Approximate Bayes factor. You tell alphaN() your sample
    size, and it tells you to which value you must lower alpha to avoid
    Lindley's Paradox. For details, see Wulff and Taylor (2024)
    <doi:10.1177/14761270231214429>. Alpha can also be calibrated to the
    effect-size and moment Bayes factors of Klauer, Meyer-Grant, and
    Kellen (2024) <doi:10.3758/s13423-024-02612-2>, which center the
    alternative hypothesis on an effect size of your choosing.
License: MIT + file LICENSE
URL: https://github.com/jespernwulff/alphaN,
        https://jespernwulff.github.io/alphaN/
BugReports: https://github.com/jespernwulff/alphaN/issues
Depends: R (>= 4.0)
Suggests: knitr, rmarkdown, spelling, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-07-11 20:52:38 UTC; au205643
Author: Jesper Wulff [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7976-0939>),
  Luke Taylor [aut]
Maintainer: Jesper Wulff <jwulff@econ.au.dk>
Repository: CRAN
Date/Publication: 2026-07-12 08:40:14 UTC

# Generated by roxygen2: do not edit by hand

export(JAB)
export(JAB_plot)
export(JABp)
export(JABt)
export(alphaN)
export(alphaN_plot)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,points)
importFrom(stats,dnorm)
importFrom(stats,dt)
importFrom(stats,integrate)
importFrom(stats,nobs)
importFrom(stats,pchisq)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,uniroot)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# alphaN <img src="man/figures/logo.png" align="right" height="120" />

<!-- badges: start -->

[![CRAN
Version](https://www.r-pkg.org/badges/version/alphaN)](https://cran.r-project.org/package=alphaN)
[![Downloads](https://cranlogs.r-pkg.org/badges/alphaN?color=brightgreen)](https://CRAN.R-project.org/package=alphaN)
[![R-CMD-check](https://github.com/jespernwulff/alphaN/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/jespernwulff/alphaN/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

The goal of alphaN is to help the user set their significance level as a
function of the sample size. The function `alphaN` allows users to set
the significance level as function of the sample size based on the
evidence and the prior features they desire. The function `JABt` and
`JABp` converts test statistics and $p$-values into sample size
dependent Bayes factors. `JAB_plot` plots the Bayes factor as a function
of the $p$-value, and `alphaN_plot` plots the alpha level as a function
of sample size for a given Bayes factor.

Calculations are based on [Wulff & Taylor
(2024)](https://doi.org/10.1177/14761270231214429). If you enjoy the
package, please consider citing the paper (see `citation("alphaN")`).

As of version 0.2.0, `alphaN()` can also calibrate the alpha level to
the effect-size and moment Bayes factors of [Klauer, Meyer-Grant &
Kellen (2024)](https://doi.org/10.3758/s13423-024-02612-2), which center
the alternative hypothesis on an effect size of your choosing
(`method = "ES"` and `method = "moment"`).

If you’re not an R user, you may also be interested in the associated
[Shiny app](https://crossvalidated.shinyapps.io/alphaN/).

## Installation

To install the latest release version from CRAN use:

``` r
install.packages("alphaN")
```

You can install the development version of alphaN from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("jespernwulff/alphaN")
```

## Examples

Here is an example: We are planning to run a linear regression model
with 1000 observations. We thus set `n = 1000`. The default `BF` is 1
meaning that we want to avoid Lindley’s paradox, i.e., we just want the
null and the alternative to be at least equally likely when we reject
the null.

``` r
library(alphaN)

alpha <- alphaN(n = 1000, BF = 1)
alpha
#> [1] 0.008582267
```

Therefore, to obtain evidence of at least 1, we should set our alpha to
0.0086.

### Targeting stronger evidence, and choosing the prior

Raising `BF` asks for more evidence before you are allowed to reject,
which lowers alpha. The `method` argument selects the prior behind
Jeffreys’ approximate Bayes factor:

``` r
# Moderate (BF = 3) and strong (BF = 10) evidence at n = 1000
alphaN(n = 1000, BF = 3)
#> [1] 0.002549145
alphaN(n = 1000, BF = 10)
#> [1] 0.0006911392

# Balancing Type I and Type II error rates instead of the default prior
alphaN(n = 1000, BF = 3, method = "balanced")
#> [1] 0.024221
```

### Calibrating alpha to effect-size and moment Bayes factors

The methods `"ES"` and `"moment"` (new in 0.2.0) answer the question:
which alpha do I need so that a significant result corresponds to a
Bayes factor of at least `BF` against an alternative centered on the
effect size `de` I actually care about?

``` r
# Moderate evidence, targeting a medium-sized effect (Cohen's d = 0.5)
alphaN(1000, BF = 3, method = "ES", de = 0.5)
#> [1] 0.002189564
alphaN(1000, BF = 3, method = "moment", de = 0.5)
#> [1] 0.0004913521
```

Because the moment prior treats effects near zero as a priori
implausible, the alpha it implies falls much faster with the sample size
than under JAB:

``` r
ns <- c(100, 1000, 10000)
tab <- rbind(JAB    = alphaN(ns, BF = 3),
             ES     = alphaN(ns, BF = 3, method = "ES"),
             moment = alphaN(ns, BF = 3, method = "moment"))
colnames(tab) <- paste0("n = ", ns)
round(tab, 5)
#>        n = 100 n = 1000 n = 10000
#> JAB    0.00910  0.00255   0.00073
#> ES     0.01185  0.00219   0.00058
#> moment 0.00788  0.00049   0.00002
```

### Turning regression output into Bayes factors

`JAB()` computes Jeffreys’ approximate Bayes factor for a coefficient
directly from a fitted `lm()` or `glm()` object; `JABt()` and `JABp()`
do the same from a t-statistic or a p-value if that is all you have
(e.g., from a published paper):

``` r
set.seed(1)
d <- data.frame(x = rnorm(200), z = rnorm(200))
d$y <- 0.2 * d$x + rnorm(200)
m <- lm(y ~ x + z, data = d)

JAB(m, covariate = "x")
#> [1] 22.50664

# From summary statistics alone
JABt(n = 200, t = 2.8)
#> [1] 3.56385
JABp(n = 200, p = 0.005)
#> [1] 3.634824
```

### Visualizing the trade-offs

`alphaN_plot()` compares alpha as a function of sample size across the
four JAB-type priors:

``` r
alphaN_plot(BF = 3)
```

<img src="man/figures/README-alphaN-plot-1.png" alt="" width="100%" />

`JAB_plot()` shows how the Bayes factor maps onto the p-value for a
given sample size, marking the alpha levels needed for evidence
thresholds of 1, 3, and 10:

``` r
JAB_plot(n = 1000, BF = 3)
```

<img src="man/figures/README-JAB-plot-1.png" alt="" width="100%" />

# Helper functions

#' Choose the prior fraction b
#'
#' Selects b, the fraction of the data used to specify the prior, for a given
#' sample size and method. Vectorized over n.
#'
#' @param n Sample size. A positive numeric vector.
#' @param method One of "JAB", "min", "robust" or "balanced".
#' @param upper Upper limit for the range of realistic effect sizes
#'   (method = "balanced" only).
#' @return A numeric vector of the same length as n.
#' @importFrom stats integrate
#' @noRd
choose_b <- function(n, method = c("JAB", "min", "robust", "balanced"),
                     upper = 1) {
  method <- match.arg(method)
  if (!is.numeric(n) || length(n) == 0 || !all(is.finite(n)) || any(n <= 0)) {
    stop("`n` must be a positive, finite numeric vector.", call. = FALSE)
  }
  if (!is.numeric(upper) || length(upper) != 1 || !is.finite(upper) ||
      upper <= 0) {
    stop("`upper` must be a single positive number.", call. = FALSE)
  }

  switch(method,
    JAB = 1/n,
    min = 2/n,
    robust = pmax(2/n, 1/sqrt(n)),
    balanced = vapply(n, function(ni) {
      max(2/ni, min(0.5,
                    integrate(function(x) exp(-ni*x^2/4), lower = 0, upper = upper)$value))
    }, numeric(1))
  )
}

#' Transforms a t-statistic from a glm or lm object into Jeffreys' approximate Bayes factor
#'
#' Extracts the test statistic of a coefficient from a fitted model object and
#' converts it into Jeffreys' approximate Bayes factor, given the sample size
#' used in the fit.
#'
#' @inheritParams JABt
#' @param glm_obj a glm or lm object.
#' @param covariate the name of the covariate that you want a BF for as a string.
#'
#' @return A numeric value for the BF in favour of H1.
#' @export
#'
#' @examples
#' # Simulate data
#'
#' ## Sample size
#' n <- 200
#'
#' ## Regressors
#' Z1 <- runif(n, -1, 1)
#' Z2 <- runif(n, -1, 1)
#' Z3 <- runif(n, -1, 1)
#' Z4 <- runif(n, -1, 1)
#' X <- runif(n, -1, 1)
#'
#' ## Error term
#' U <- rnorm(n, 0, 0.5)
#'
#' ## Outcome
#' Y <- X/sqrt(n) + U
#'
#' # Run a GLM
#' LM <- glm(Y ~ X + Z1 + Z2 + Z3 + Z4)
#'
#' # Compute JAB for "X" based on the regression results
#' JAB(LM, "X")
#'
#' # Compute JAB using the minimum prior
#' JAB(LM, "X", method = "min")
#' @importFrom stats nobs
JAB <- function(glm_obj, covariate, method="JAB", upper = 1){
  if (!inherits(glm_obj, c("glm", "lm"))) {
    stop("`glm_obj` must be a model fitted with lm() or glm().", call. = FALSE)
  }
  if (!is.character(covariate) || length(covariate) != 1) {
    stop("`covariate` must be a single character string.", call. = FALSE)
  }

  coefs <- summary(glm_obj)$coefficients
  if (!covariate %in% rownames(coefs)) {
    stop("Covariate '", covariate, "' not found in the model. Available: ",
         paste(rownames(coefs), collapse = ", "), ".", call. = FALSE)
  }

  n <- nobs(glm_obj)
  t <- coefs[covariate, 3]

  BF <- JABt(n = n, t = t, method = method, upper = upper)

  return(BF)
}

test_that("JAB correctly extracts t-stat", {
  n <- 200
  set.seed(200)
  ## Regressors
  Z1 <- runif(n, -1, 1)
  Z2 <- runif(n, -1, 1)
  Z3 <- runif(n, -1, 1)
  Z4 <- runif(n, -1, 1)
  X <- runif(n, -1, 1)
  ## Error term
  U <- rnorm(n, 0, 0.5)
  ## Outcome
  Y <- X/sqrt(n) + U
  # Run a GLM
  LM <- glm(Y ~ X + Z1 + Z2 + Z3 + Z4)
  # Compute JAB for "X" based on the regression results
  JAB_obj <- JAB(LM, "X")
  # check that JAB has extracted the correct t-statistic
  t_X <- summary(LM)$coefficients["X", 3]
  expect_equal(JAB_obj, JABt(n = n, t = t_X))
  # an lm fit of the same model gives the same result
  expect_equal(JAB(lm(Y ~ X + Z1 + Z2 + Z3 + Z4), "X"), JAB_obj)
})

test_that("JAB uses the number of observations actually used in the fit", {
  set.seed(1)
  d <- data.frame(Y = rnorm(50), X = rnorm(50))
  d$X[1:5] <- NA
  m <- lm(Y ~ X, data = d)
  expect_equal(JAB(m, "X"),
               JABt(n = nobs(m), t = summary(m)$coefficients["X", 3]))
})

test_that("JAB works for non-gaussian glms", {
  set.seed(2)
  X <- rnorm(100)
  Y <- rbinom(100, 1, plogis(0.5 * X))
  m <- glm(Y ~ X, family = binomial)
  expect_equal(JAB(m, "X"),
               JABt(n = 100, t = summary(m)$coefficients["X", 3]))
})

test_that("JAB rejects invalid input with informative errors", {
  set.seed(3)
  d <- data.frame(Y = rnorm(20), X = rnorm(20))
  m <- lm(Y ~ X, data = d)
  expect_error(JAB(d, "X"), "lm\\(\\) or glm\\(\\)")
  expect_error(JAB(m, "Z"), "not found")
  expect_error(JAB(m, "Z"), "Available")
  expect_error(JAB(m, c("X", "Y")), "single character string")
  expect_error(JAB(m, 1), "single character string")
})

#' Plots JAB as a function of the p-value
#'
#' @inheritParams JABt
#' @param BF Bayes factor you would like to match. 1 to avoid the Lindley Paradox, 3 to achieve moderate evidence and 10 to achieve strong evidence.
#'
#' @return Prints a plot.
#' @export
#'
#' @examples
#' # Plot JAB as function of the p-value for a sample size of 2000
#' JAB_plot(2000)
#' @importFrom graphics abline axis points
JAB_plot <- function(n, BF=1, method="JAB", upper = 1){
  method <- match.arg(method, c("JAB", "min", "robust", "balanced"))

  alpha <- alphaN(n = n, BF = BF, method = method, upper = upper)

  indicated <- alpha
  lindley <- alphaN(n = n, BF = 1, method = method, upper = upper)
  moderate <- alphaN(n = n, BF = 3, method = method, upper = upper)
  strong <- alphaN(n = n, BF = 10, method = method, upper = upper)

  ### and make a sequence of t-stats to get p-values
  ts <- seq(1.2, 5, length.out=150)
  ps <- 1-pchisq(ts^2, 1)
  ## Third, get a Bayes factor for every p-value
  bf <- JABt(n = n, t = ts, method = method, upper = upper)

  plot(ps, bf, type="l", lty=1, lwd=2, log = "y",
       xlab = bquote(italic("p")*"-value"), ylab = "Bayes factor",
       ylim = c(0.1, 10), xlim = c(0, max(c(0.05, lindley))),
       axes=FALSE)

  axis(side=1, at = c(0, as.numeric(lindley), as.numeric(moderate),
                      as.numeric(strong), 0.05, indicated),
       labels = c(0, round(lindley, digits = 3), round(moderate, digits = 3),
                  round(strong, digits = 3), 0.05, round(indicated, digits = 3)),
       lwd = 2, las = 3)

  axis(side=2, at = c(0.1, 0.33, 1, 3, 10), labels = c("1/10", "1/3", 1, 3, 10), lwd = 2)
  abline(h = c(0.1, 0.33, 1, 3, 10), col = "gray", lty = 2)
  abline(v = c(lindley, moderate, strong), lty = 3)
  abline(v = indicated, lty = 3, col = "red")

  points(indicated, BF, col = "black", bg="red", cex=2, pch = 24, lwd = 2)
}

test_that("plot functions run without errors or warnings", {
  pdf(NULL)
  on.exit(dev.off())
  expect_silent(alphaN_plot(BF = 3, max = 1000))
  expect_silent(alphaN_plot(BF = 1, max = 500, ylim = c(0, 0.05)))
  expect_silent(JAB_plot(1000))
  expect_silent(JAB_plot(500, BF = 3, method = "robust"))
  expect_silent(JAB_plot(500, BF = 3, method = "balanced", upper = 0.5))
})

# Tests for the effect-size ("ES") and moment Bayes factor methods of
# Klauer, Meyer-Grant & Kellen (2024, Psychonomic Bulletin & Review).

test_that("Bayes factors reproduce Table 7 of Klauer et al. (2024)", {
  # One-sample t tests with t(79), i.e. n = 80, from Grider & Malmberg data.
  # Printed values are rounded to two decimals -> tolerance 0.006. The ES cell
  # at t = 2.24, de = 0.5 gets 0.015: we compute 1.568 (i.e. 1.57), while the
  # paper prints 1.58. The other eleven cells and the JZS special case all
  # match to print rounding at t = 2.24 (and no in-rounding t reconciles all
  # cells at once), so the printed 1.58 appears to be off in its last digit.
  tab <- expand.grid(t = c(2.03, 2.24), de = c(0.2, 0.5, 0.8))
  tab$es  <- c(2.52, 3.70, 0.98, 1.58, 0.51, 0.80)
  tab$mom <- c(2.13, 3.26, 0.58, 0.99, 0.19, 0.34)
  tab$tol <- c(0.006, 0.006, 0.006, 0.015, 0.006, 0.006)
  for (i in seq_len(nrow(tab))) {
    es <- klauer_bf(tab$t[i], 80, "ES", de = tab$de[i], nu = 3,
                    r = sqrt(1/3)*tab$de[i])
    mo <- klauer_bf(tab$t[i], 80, "moment", de = tab$de[i], nu = 5)
    expect_lt(abs(es - tab$es[i]), tab$tol[i])
    expect_lt(abs(mo - tab$mom[i]), 0.006)
  }
})

test_that("nu = 1, de = 0 recovers the default (JZS-type) Bayes factor", {
  # Rouder et al. (2009) values with r = 1, printed in Table 7 of Klauer et al.
  expect_lt(abs(klauer_bf(2.03, 80, "ES", de = 0, nu = 1, r = 1) - 0.64), 0.006)
  expect_lt(abs(klauer_bf(2.24, 80, "ES", de = 0, nu = 1, r = 1) - 0.98), 0.006)
})

test_that("alphaN methods ES and moment invert their Bayes factors", {
  for (m in c("ES", "moment")) {
    for (bf in c(1, 3)) {
      a <- alphaN(500, BF = bf, method = m, de = 0.5)
      t_crit <- qt(1 - a/2, df = 499)
      nu <- if (m == "ES") 3 else 5
      r <- if (m == "ES") sqrt(1/3)*0.5 else NULL
      expect_equal(klauer_bf(t_crit, 500, m, de = 0.5, nu = nu, r = r), bf,
                   tolerance = 1e-4)
    }
  }
})

test_that("ES and moment alphas decrease in n and BF, vectorized", {
  ns <- c(100, 1000)
  for (m in c("ES", "moment")) {
    a_vec <- alphaN(ns, BF = 3, method = m)
    a_ele <- vapply(ns, function(n) alphaN(n, BF = 3, method = m), numeric(1))
    expect_equal(a_vec, a_ele)
    expect_true(all(diff(a_vec) < 0))
    expect_gt(alphaN(1000, BF = 1, method = m), alphaN(1000, BF = 10, method = m))
  }
})

test_that("moment alpha falls much faster in n than JAB alpha", {
  expect_lt(alphaN(10000, BF = 1, method = "moment", de = 0.5),
            alphaN(10000, BF = 1)/10)
})

test_that("large-n normal limit joins the exact-t branch smoothly", {
  # .klauer_normal_switch is 50000; compare alphas just below and above
  a_lo <- alphaN(49000, BF = 3, method = "ES", de = 0.5)
  a_hi <- alphaN(51000, BF = 3, method = "ES", de = 0.5)
  expect_lt(abs(a_hi/a_lo - 1), 0.05)
  expect_lt(a_hi, a_lo)  # still decreasing across the switch
})

test_that("ES/moment input validation gives informative errors", {
  expect_error(alphaN(1000, method = "ES", de = -0.2), "de")
  expect_error(alphaN(1000, method = "moment", de = 0), "de")
  expect_error(alphaN(1000, method = "moment", nu = 2), "nu")
  expect_error(alphaN(1000, method = "ES", nu = 1), "supply the prior scale")
  expect_error(alphaN(1000, method = "ES", de = 0), "supply the prior scale")
  expect_error(alphaN(1000, method = "ES", r = -1), "r")
  expect_error(alphaN(3, method = "ES"), "at least")
  expect_error(alphaN(-10, method = "ES"), "positive")
  # methods "ES"/"moment" are not available in the JAB-family functions
  expect_error(JABt(100, 2, method = "ES"), "should be one of")
  expect_error(JAB_plot(100, method = "ES"), "should be one of")
})

test_that("alpha = 1 when even t = 0 exceeds the evidence target", {
  # At tiny BF targets the Bayes factor at t = 0 can already exceed BF
  expect_equal(alphaN(100, BF = 0.01, method = "ES", de = 0.5), 1)
})

#' Creates a plot of alpha as function of sample size for each of the four prior options
#'
#' @inheritParams alphaN
#' @param max The maximum number of sample size. Defaults to 10,000.
#' @param ylim Limits for the y-axis. The default, NULL, covers all four
#'   curves. Set to e.g. c(0, 0.05) to zoom in on small alpha levels.
#'
#' @return Prints a plot.
#' @export
#'
#' @examples
#' # Plot of alpha level as a function of n for a Bayes factor of 3
#' alphaN_plot(BF = 3)
#' @importFrom graphics legend lines
alphaN_plot <- function(BF = 1, max = 10000, ylim = NULL){
  seqN <- seq(50, max, 1)

  alpha_jab <- alphaN(seqN, BF = BF)
  alpha_min <- alphaN(seqN, BF = BF, method = "min")
  alpha_robust <- alphaN(seqN, BF = BF, method = "robust")
  alpha_balanced <- alphaN(seqN, BF = BF, method = "balanced")

  if (is.null(ylim)) {
    ylim <- c(0, base::max(alpha_jab, alpha_min, alpha_robust, alpha_balanced))
  }

  plot(seqN, alpha_jab, type = "l",
       xlab = "Sample size", ylab = expression(paste(alpha)),
       lwd=2, ylim= ylim, xlim=c(0,max),
       axes=FALSE,
       col="#F8766D",
       main = paste("Bayes factor = ", BF))
  lines(seqN, alpha_min, lty=2, lwd=2, col="#7CAE00")
  lines(seqN, alpha_robust, lty=3, lwd=2, col="#00BFC4")
  lines(seqN, alpha_balanced, lty=4, lwd=2, col= "#C77CFF")
  axis(side = 1, at = pretty(c(0, max)))
  axis(side = 2, at = pretty(ylim))

  legend("topright",
         c("JAB",
           "Min",
           "Robust",
           "Balanced"
         ),
         lty = c(1,2,3,4),
         lwd = c(2,2,2,2),
         col = c("#F8766D",
                 "#7CAE00",
                 "#00BFC4",
                 "#C77CFF"),
         box.col="white", bg="white")

}

#' Transforms a t-statistic into Jeffreys' approximate Bayes factor
#'
#' Converts a t-statistic (or z-statistic) into Jeffreys' approximate Bayes
#' factor, given the sample size. Vectorized over `n` and `t`.
#'
#' @param n Sample size. A positive numeric vector.
#' @param t The t-statistic.
#' @param method Used for the choice of 'b'. Currently one of:
#' \itemize{
#'   \item "JAB": this choice of b produces Jeffreys' approximate BF (Wagenmakers, 2022)
#'   \item "min": uses the minimal training sample for the prior (Gu et al., 2018)
#'   \item "robust": a robust version of "min" that prevents too small b (O'Hagan, 1995)
#'   \item "balanced": this choice of b balances the type I and type II errors (Gu et al., 2016)
#' }
#' @param upper The upper limit for the range of realistic effect sizes. Only relevant when method="balanced". Defaults to 1 such that the range of realistic effect sizes is uniformly distributed between 0 and 1, U(0,1).
#' @return A numeric value for the BF in favour of H1.
#' @export
#'
#' @examples
#' # Transform a t-statistic of 2.695 computed based on a sample size of 200 into JAB
#' JABt(200, 2.695)
JABt <- function(n, t, method = "JAB", upper = 1){
  if (!is.numeric(t) || length(t) == 0 || anyNA(t)) {
    stop("`t` must be a numeric vector without missing values.", call. = FALSE)
  }

  b <- choose_b(n, method, upper)

  BF <- exp(0.5*t^2)*sqrt(b)

  return(BF)
}

test_that("JABt is consistent with alphaN", {
  # calculate the p-value for a z-score of 1.96
  z <- 1.96
  p <- 2*pnorm(q=z, lower.tail=FALSE)

  # compute BF for z-score assuming a sample size of 100
  n <- 100
  BF <- JABt(n, z)

  # compute alpha for that BF and sample size
  alpha <- alphaN(n, BF = BF)

  # check that the result from alphaN matches the p-value
  expect_equal(alpha, p)

  # repeat for other methods
  ## min
  BF <- JABt(n, z, method = "min")
  alpha <- alphaN(n, BF = BF, method = "min")
  expect_equal(alpha, p)

  ## robust
  BF <- JABt(n, z, method = "robust")
  alpha <- alphaN(n, BF = BF, method = "robust")
  expect_equal(alpha, p)

  ## balanced
  BF <- JABt(n, z, method = "balanced")
  alpha <- alphaN(n, BF = BF, method = "balanced")
  expect_equal(alpha, p)

})

test_that("larger sample size decreases BF for constant t-score", {
  expect_gt(JABt(100, 1), JABt(101, 1))
  expect_gt(JABt(200, 1), JABt(201, 1))
  expect_gt(JABt(3000, 1), JABt(3001, 1))
})

test_that("larger t-score increases BF for fixed n", {
  expect_gt(JABt(100, 0.01), JABt(100, 0))
  expect_gt(JABt(100, 1.01), JABt(100, 1))
  expect_gt(JABt(100, 2.01), JABt(100, 2))
})

test_that("method = JAB has the lowest BF", {
  expect_gt(JABt(100, 1, method = "balanced"), JABt(100, 1, method = "JAB"))
  expect_gt(JABt(100, 1, method = "robust"), JABt(100, 1, method = "JAB"))
  expect_gt(JABt(100, 1, method = "min"), JABt(100, 1, method = "JAB"))
})

test_that("method = balanced has the highest BF", {
  expect_gt(JABt(100, 1, method = "balanced"), JABt(100, 1, method = "robust"))
  expect_gt(JABt(100, 1, method = "balanced"), JABt(100, 1, method = "min"))
})

test_that("method = robust has a higher BF than method = min", {
  expect_gt(JABt(100, 1, method = "robust"), JABt(100, 1, method = "min"))
  expect_gt(JABt(200, 1, method = "robust"), JABt(200, 1, method = "min"))
  expect_gt(JABt(300, 1, method = "robust"), JABt(300, 1, method = "min"))
})

test_that("JABt vectorizes over n for every method", {
  ns <- c(100, 1000, 10000)
  for (m in c("JAB", "min", "robust", "balanced")) {
    expect_equal(JABt(ns, 2, method = m),
                 vapply(ns, function(n) JABt(n, 2, method = m), numeric(1)))
  }
})

test_that("JABt vectorizes over t", {
  ts <- c(0, 1, 2.5)
  expect_equal(JABt(100, ts),
               vapply(ts, function(t) JABt(100, t), numeric(1)))
})

test_that("JABt rejects invalid input with informative errors", {
  expect_error(JABt(100, 2, method = "typo"), "should be one of")
  expect_error(JABt(-100, 2), "positive")
  expect_error(JABt(100, "big"), "numeric")
  expect_error(JABt(100, NA), "missing")
})

#' Transforms a p-value into Jeffreys' approximate Bayes factor
#'
#' Converts a two-sided p-value from a z- or t-test into Jeffreys' approximate
#' Bayes factor, given the sample size.
#'
#' @inheritParams JABt
#' @param p The two-sided p-value.
#' @param z Is the p-value based on a z- or t-statistic? TRUE if z.
#' @param df If z=FALSE, provide the degrees of freedom for the t-statistic.
#'
#' @return A numeric value for the BF in favour of H1.
#' @export
#'
#' @examples
#' # Transform a p-value of 0.007038863 from a z-test into JAB
#' # using a sample size of 200.
#' JABp(200, 0.007038863)
#'
#' # Transform a p-value of 0.007038863 from a t-test with 190
#' # degrees of freedom into JAB using a sample size of 200.
#' JABp(200, 0.007038863, z=FALSE, df=190)
#'
#' @importFrom stats qnorm qt
JABp <- function(n, p, z = TRUE, df = NULL, method = "JAB", upper = 1){
  if (!is.numeric(p) || length(p) == 0 || !all(is.finite(p)) ||
      any(p <= 0) || any(p > 1)) {
    stop("`p` must be numeric with values in (0, 1].", call. = FALSE)
  }
  if (!isTRUE(z) && !isFALSE(z)) {
    stop("`z` must be TRUE or FALSE.", call. = FALSE)
  }

  # Find the corresponding z- or t-statistic
  if(z){
    stat <- qnorm(p/2, lower.tail = FALSE)
  } else {
    if (is.null(df)) {
      stop("When `z = FALSE`, provide the degrees of freedom of the t-statistic via `df`.",
           call. = FALSE)
    }
    if (!is.numeric(df) || length(df) == 0 || !all(is.finite(df)) ||
        any(df <= 0)) {
      stop("`df` must be a positive, finite numeric value.", call. = FALSE)
    }
    stat <- qt(p/2, df = df, lower.tail = FALSE)
  }

  BF <- JABt(n, stat, method = method, upper = upper)

  return(BF)
}

test_that("JABp is consistent with alphaN", {
  BF <- 1
  n <- 100

  # JAB
  alpha <- alphaN(n, BF = BF)
  expect_equal(JABp(n, alpha), BF)

  # min
  alpha <- alphaN(n, BF = BF, method = "min")
  expect_equal(JABp(n, alpha, method = "min"), BF)

  # robust
  alpha <- alphaN(n, BF = BF, method = "robust")
  expect_equal(JABp(n, alpha, method = "robust"), BF)

  # balanced
  alpha <- alphaN(n, BF = BF, method = "balanced")
  expect_equal(JABp(n, alpha, method = "balanced"), BF)
})

test_that("JAB increases when df decrease for the t-stat", {
  n <- 100
  p <- 0.05
  expect_gt(JABp(n, p, z = FALSE, df = 99), JABp(n, p, z = FALSE, df = 100))
  expect_gt(JABp(n, p, z = FALSE, df = 98), JABp(n, p, z = FALSE, df = 99))
  expect_gt(JABp(n, p, z = FALSE, df = 97), JABp(n, p, z = FALSE, df = 98))
})

test_that("JABp rejects invalid input with informative errors", {
  expect_error(JABp(100, 0.05, z = FALSE), "df")
  expect_error(JABp(100, 0.05, z = FALSE, df = -3), "df")
  expect_error(JABp(100, 1.5), "\\(0, 1\\]")
  expect_error(JABp(100, 0), "\\(0, 1\\]")
  expect_error(JABp(100, -0.2), "\\(0, 1\\]")
  expect_error(JABp(100, "small"), "numeric")
  expect_error(JABp(100, 0.05, z = "yes"), "TRUE or FALSE")
  expect_error(JABp(100, 0.05, method = "typo"), "should be one of")
})

#' Set the alpha level based on sample size for coefficients in a regression model
#'
#' Computes the alpha level required to achieve a desired level of evidence,
#' expressed as a Bayes factor, when testing a coefficient in a regression
#' model. The alpha level is a decreasing function of the sample size.
#' Vectorized over `n` and `BF`.
#'
#' @param n Sample size. A positive numeric vector.
#' @param BF Bayes factor you would like to match. 1 to avoid Lindley's Paradox, 3 to achieve moderate evidence and 10 to achieve strong evidence.
#' @param method Which Bayes factor to calibrate alpha to. The first four
#' options invert Jeffreys' approximate Bayes factor and differ in the choice
#' of the prior fraction 'b'; the last two invert the exact test-statistic
#' Bayes factors of Klauer et al. (2024), whose priors center the alternative
#' hypothesis on a prespecified effect size `de`. One of:
#' \itemize{
#'   \item "JAB": this choice of b produces Jeffreys' approximate BF (Wagenmakers, 2022)
#'   \item "min": uses the minimal training sample for the prior (Gu et al., 2018)
#'   \item "robust": a robust version of "min" that prevents too small b (O'Hagan, 1995)
#'   \item "balanced": this choice of b balances the type I and type II errors (Gu et al., 2016)
#'   \item "ES": calibrates alpha to the effect-size Bayes factor (Klauer et al., 2024)
#'   \item "moment": calibrates alpha to the moment Bayes factor (Klauer et al., 2024), under which effects close to zero are a priori implausible
#' }
#' @param upper The upper limit for the range of realistic effect sizes. Only relevant when method="balanced". Defaults to 1 such that the range of realistic effect sizes is uniformly distributed between 0 and 1, U(0,1).
#' @param de The prespecified (targeted) effect size in standardized units
#'   (Cohen's d). Only used by methods "ES" and "moment". Defaults to 0.5, a
#'   medium effect; use 0.2 for small and 0.8 for large effects (Cohen, 1988).
#' @param nu Degrees of freedom of the prior t distribution for methods "ES"
#'   and "moment". The default, NULL, uses the values recommended by Klauer
#'   et al. (2024): 3 for "ES" and 5 for "moment".
#' @param r Scale of the two prior mixture components for method "ES". The
#'   default, NULL, uses the recommendation of Klauer et al. (2024),
#'   r = sqrt((nu - 2)/nu) * de, which requires nu > 2 and de > 0; otherwise
#'   supply `r` explicitly.
#' @return Numeric alpha level required to achieve the desired level of evidence.
#'
#' @details
#' For methods "ES" and "moment", the alpha level is found by solving for the
#' critical t value at which the effect-size or moment Bayes factor equals
#' `BF`, and converting that critical value to a two-sided p-value on the t
#' distribution with `n - 1` degrees of freedom (the one-sample /
#' single-coefficient case of Klauer et al., 2024). The implementation is
#' validated against the Bayes factors reported in Table 7 of that paper.
#' Because the moment prior assigns effects near zero a priori density zero,
#' the alpha level it implies decreases much faster with `n` than under JAB.
#'
#' As a special case, setting \code{method = "ES", nu = 1, de = 0} with an
#' explicit scale (e.g. \code{r = 1}) calibrates alpha to the default
#' (Jeffreys-Zellner-Siow type) Bayes factor of Rouder et al. (2009).
#'
#' For `n` greater than 50,000, methods "ES" and "moment" evaluate the
#' noncentral-t density ratio in its normal limit, which is accurate to a
#' fraction of a percent there.
#' @export
#'
#' @examples
#'# Plot of alpha level as a function of n
#'seqN <- seq(50, 1000, 1)
#'plot(seqN, alphaN(seqN), type = "l")
#'
#'# Alpha calibrated to the effect-size Bayes factor (Klauer et al., 2024),
#'# targeting moderate evidence for a medium-sized effect
#'alphaN(1000, BF = 3, method = "ES", de = 0.5)
#'
#'# The same calibration under the moment Bayes factor
#'alphaN(1000, BF = 3, method = "moment", de = 0.5)
#' @section References:
#' Gu et al. (2016). Error probabilities in default Bayesian hypothesis testing. Journal of Mathematical Psychology, 72, 130–143. \cr
#' \cr
#' Gu et al. (2018). Approximated adjusted fractional Bayes factors: A general method for testing informative hypotheses. The British Journal of Mathematical and Statistical Psychology, 71(2). \cr
#' \cr
#' Klauer, K. C., Meyer-Grant, C. G., & Kellen, D. (2024). On Bayes factors for hypothesis tests. Psychonomic Bulletin & Review. \doi{10.3758/s13423-024-02612-2} \cr
#' \cr
#' O’Hagan, A. (1995). Fractional Bayes Factors for Model Comparison. Journal of the Royal Statistical Society. Series B (Methodological), 57(1), 99–138. \cr
#' \cr
#' Rouder, J. N., Speckman, P. L., Sun, D., Morey, R. D., & Iverson, G. (2009). Bayesian t tests for accepting and rejecting the null hypothesis. Psychonomic Bulletin & Review, 16, 225–237. \cr
#' \cr
#' Wagenmakers, E.-J. (2022). Approximate objective Bayes factors from p-values and sample size: The 3p(sqrt(n)) rule. PsyArXiv. \cr
#' \cr
#' Wulff, J. N., & Taylor, L. (2024). How and why alpha should depend on sample size: A Bayesian-frequentist compromise for significance testing. Strategic Organization. \doi{10.1177/14761270231214429}
#' @importFrom stats pchisq
alphaN <- function(n, BF=1, method="JAB", upper = 1, de = 0.5, nu = NULL,
                   r = NULL) {
  method <- match.arg(method,
                      c("JAB", "min", "robust", "balanced", "ES", "moment"))
  if (!is.numeric(BF) || length(BF) == 0 || !all(is.finite(BF)) ||
      any(BF <= 0)) {
    stop("`BF` must be a positive, finite numeric value.", call. = FALSE)
  }

  if (method %in% c("ES", "moment")) {
    return(klauer_alpha(n, BF, method, de = de, nu = nu, r = r))
  }

  b <- choose_b(n, method, upper)

  alpha <- 1-pchisq(2*log(BF/sqrt(b)), 1)
  return(alpha)
}

test_that("larger sample size decreases alpha", {
  expect_gt(alphaN(100), alphaN(101))
  expect_gt(alphaN(200), alphaN(201))
  expect_gt(alphaN(2000), alphaN(2001))
})

test_that("larger Bayes factor decreases alpha", {
  expect_gt(alphaN(100, BF = 1), alphaN(100, BF = 2))
  expect_gt(alphaN(200, BF = 1), alphaN(200, BF = 2))
  expect_gt(alphaN(2000, BF = 1), alphaN(2000, BF = 2))
})

test_that("method = JAB has the lowest alpha", {
  expect_gt(alphaN(100, method = "balanced"), alphaN(100, method = "JAB"))
  expect_gt(alphaN(100, method = "robust"), alphaN(100, method = "JAB"))
  expect_gt(alphaN(100, method = "min"), alphaN(100, method = "JAB"))
})

test_that("method = balanced has the highest alpha", {
  expect_gt(alphaN(100, method = "balanced"), alphaN(100, method = "robust"))
  expect_gt(alphaN(100, method = "balanced"), alphaN(100, method = "min"))
})

test_that("method = robust has a higher alpha than method = min", {
  expect_gt(alphaN(100, method = "robust"), alphaN(100, method = "min"))
  expect_gt(alphaN(200, method = "robust"), alphaN(200, method = "min"))
  expect_gt(alphaN(300, method = "robust"), alphaN(300, method = "min"))
})

test_that("alphaN vectorizes over n for every method", {
  ns <- c(100, 1000, 10000)
  for (m in c("JAB", "min", "robust", "balanced")) {
    expect_equal(alphaN(ns, method = m),
                 vapply(ns, function(n) alphaN(n, method = m), numeric(1)))
  }
})

test_that("alphaN vectorizes over BF", {
  BFs <- c(1, 3, 10)
  expect_equal(alphaN(1000, BF = BFs),
               vapply(BFs, function(BF) alphaN(1000, BF = BF), numeric(1)))
})

test_that("alphaN rejects invalid input with informative errors", {
  expect_error(alphaN(1000, method = "typo"), "should be one of")
  expect_error(alphaN(-5), "positive")
  expect_error(alphaN(0), "positive")
  expect_error(alphaN(Inf), "finite")
  expect_error(alphaN("many"), "numeric")
  expect_error(alphaN(1000, BF = -2), "BF")
  expect_error(alphaN(1000, BF = 0), "BF")
  expect_error(alphaN(1000, method = "balanced", upper = -1), "upper")
})

# Effect-size and moment Bayes factors (Klauer, Meyer-Grant & Kellen, 2024,
# Psychonomic Bulletin & Review, doi:10.3758/s13423-024-02612-2) and their
# inversion to alpha levels. Internal.
#
# Both Bayes factors have the test-statistic form (Table 4 of the paper,
# one-sample / single-coefficient case):
#   B10 = Int dt(t | n - 1, ncp = sqrt(n) d) / dt(t | n - 1, 0) * pi(d) dd
# with pi(d) the prior on the standardized effect size d:
#   effect size: pi_E(d) = ( ft(d | nu, -de, r) + ft(d | nu, de, r) ) / 2
#   moment:      pi_M(d) = 2(nu-2)/((nu-1) de^2) d^2 ft(d | nu, 0, s),
#                s = de sqrt((nu-1)/(2 nu))
# where ft(. | nu, m, s) is the scaled and shifted t density.

# Scaled and shifted t density
ft_scaled <- function(x, nu, mean, scale) {
  dt((x - mean)/scale, df = nu)/scale
}

# log of dt(t | df, ncp)/dt(t | df, 0). R's noncentral t density is a
# difference of two CDF calls and can cancel to NaN deep in the tails, where
# the true ratio is effectively zero -> clamp to -Inf.
lratio_t <- function(t, df, ncp) {
  lr <- suppressWarnings(dt(t, df = df, ncp = ncp, log = TRUE) -
                           dt(t, df = df, log = TRUE))
  lr[!is.finite(lr)] <- -Inf
  lr
}

# Large-n (normal) limit of the same log ratio:
# log dnorm(t, ncp, 1) - log dnorm(t, 0, 1)
lratio_norm <- function(t, ncp) {
  t*ncp - ncp^2/2
}

# Above this sample size the noncentral t density ratio is evaluated in its
# normal limit: R's dnt carries too much numerical noise at extreme df, and
# the limit is accurate to a fraction of a percent there.
.klauer_normal_switch <- 50000

#' Effect-size or moment Bayes factor from a t-statistic
#'
#' Integrates over u = sqrt(n) d (the noncentrality parameter) so that the
#' density-ratio factor varies on an O(1) scale for every n. The integration
#' bounds +/-(|t| + 45) are conservative: beyond them the density ratio has
#' decayed below the double-precision floor.
#'
#' @param t t-statistic (scalar).
#' @param n Sample size (scalar).
#' @param method "ES" or "moment".
#' @param de Prespecified effect size.
#' @param nu Degrees of freedom of the prior.
#' @param r Scale of the prior mixture components ("ES" only).
#' @return The Bayes factor in favour of H1 (scalar).
#' @importFrom stats dt dnorm integrate
#' @noRd
klauer_bf <- function(t, n, method, de, nu, r = NULL) {
  prior <- if (method == "ES") {
    function(d) 0.5*(ft_scaled(d, nu, -de, r) + ft_scaled(d, nu, de, r))
  } else {
    s <- de*sqrt((nu - 1)/(2*nu))
    function(d) 2*(nu - 2)/((nu - 1)*de^2) * d^2 * ft_scaled(d, nu, 0, s)
  }

  sn <- sqrt(n)
  lim <- abs(t) + 45
  kernel <- function(lr_fun) {
    integrate(function(u) exp(lr_fun(u)) * prior(u/sn)/sn,
              lower = -lim, upper = lim,
              rel.tol = 1e-6, subdivisions = 500L)$value
  }

  if (n > .klauer_normal_switch) {
    return(kernel(function(u) lratio_norm(t, u)))
  }
  tryCatch(suppressWarnings(kernel(function(u) lratio_t(t, n - 1, u))),
           error = function(e) kernel(function(u) lratio_norm(t, u)))
}

#' Validate and resolve the ES/moment prior settings
#'
#' Fills in the recommended defaults of Klauer et al. (2024): nu = 3 and
#' r = sqrt((nu - 2)/nu) de for "ES", nu = 5 for "moment".
#'
#' @return list(nu = ..., r = ...)
#' @noRd
resolve_klauer_args <- function(n, method, de, nu, r) {
  if (!is.numeric(de) || length(de) != 1 || !is.finite(de) || de < 0) {
    stop("`de` must be a single non-negative number.", call. = FALSE)
  }
  if (is.null(nu)) nu <- if (method == "ES") 3 else 5
  if (!is.numeric(nu) || length(nu) != 1 || !is.finite(nu)) {
    stop("`nu` must be a single number.", call. = FALSE)
  }

  if (method == "moment") {
    if (de == 0) {
      stop("`de` must be positive for method = \"moment\".", call. = FALSE)
    }
    if (nu <= 2) {
      stop("`nu` must be larger than 2 for method = \"moment\".", call. = FALSE)
    }
  } else {
    if (nu < 1) {
      stop("`nu` must be at least 1 for method = \"ES\".", call. = FALSE)
    }
    if (is.null(r)) {
      if (nu <= 2 || de == 0) {
        stop("For method = \"ES\" with `nu` <= 2 or `de` = 0, supply the prior scale `r` explicitly.",
             call. = FALSE)
      }
      r <- sqrt((nu - 2)/nu)*de
    }
    if (!is.numeric(r) || length(r) != 1 || !is.finite(r) || r <= 0) {
      stop("`r` must be a single positive number.", call. = FALSE)
    }
  }

  if (!is.numeric(n) || length(n) == 0 || !all(is.finite(n)) || any(n <= 0)) {
    stop("`n` must be a positive, finite numeric vector.", call. = FALSE)
  }
  n_min <- if (method == "ES") nu + 1 else max(nu - 1, 2)
  if (any(n < n_min)) {
    stop("For method = \"", method, "\" with nu = ", nu, ", `n` must be at least ",
         n_min, " for the Bayes factor to be invertible (consistency in information; ",
         "Klauer et al., 2024).", call. = FALSE)
  }

  list(nu = nu, r = r)
}

#' Alpha level calibrated to the effect-size or moment Bayes factor
#'
#' Solves for the critical t at which the Bayes factor equals `BF` and
#' converts it to a two-sided p-value on the t distribution with n - 1
#' degrees of freedom. Vectorized over `n` and `BF` (recycled).
#'
#' @return Numeric vector of alpha levels.
#' @importFrom stats pt uniroot
#' @noRd
klauer_alpha <- function(n, BF, method, de, nu, r) {
  args <- resolve_klauer_args(n, method, de, nu, r)

  len <- max(length(n), length(BF))
  n <- rep_len(n, len)
  BF <- rep_len(BF, len)

  vapply(seq_len(len), function(i) {
    bf_i <- function(t) klauer_bf(t, n[i], method, de, args$nu, args$r)
    # If even t = 0 meets the evidence target, any result does: alpha = 1
    if (bf_i(0) >= BF[i]) return(1)
    t_crit <- uniroot(function(t) log(bf_i(t)) - log(BF[i]),
                      interval = c(0, 12), extendInt = "upX", tol = 1e-8)$root
    2*pt(t_crit, df = n[i] - 1, lower.tail = FALSE)
  }, numeric(1))
}

---
title: "Intro to alphaN"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Intro to alphaN}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(alphaN)
```

We wish to determine which alpha level is equivalent to a Bayes factor of 1. I.e. only reject the null if the data is at least at likely under the null and under the alternative. To do this, we need a way to connect the $p$-value to the Bayes factor. The **alphaN** package does this for tests of coefficients in regression models. 

# Installation

You can install the development version of alphaN from [GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("jespernwulff/alphaN")
```

# Basic functionality

This vignette provides an introduction to the basic functionality of **alphaN**. For full details on methodology, please refer to Wulff & Taylor (2023).

## Setting the alpha level

Using the `alphaN` function, we can get the alpha level we need to use to obtain a desired level of evidence when testing a regression coefficient in regression model.

Here is an example: We are planning to run a linear regression model with 1000 observations. We thus set `n = 1000`. The default `BF` is 1 meaning that we want to avoid Lindley's paradox, i.e. we just want the null and the alternative to be at least equally likely when we reject the null. 

```{r}
alpha <- alphaN(n = 1000, BF = 1)
alpha
```

Therefore, to obtain evidence of at least 1, we should set our alpha to `r round(alpha,4)`.

## Plotting the relationship between the Bayes factor and *p*-value

The `alphaN` function works by mapping the *p*-value to the Bayes factor. This relationship can be shown using the `JAB_plot`. For instance:

```{r}
JAB_plot(n = 1000, BF = 1)
```

The alpha level needed to achieve a Bayes factor of 1 is shown with a red triangle in the plot. Lines for achieving Bayes factors of 3 (moderate evidence) and 10 (strong evidence) are also shown by default. As it is evident a lower alpha level is needed to achieve higher evidence.

## Alpha as a decreasing function of N

An important point of the procedure is that alpha will be set as a function of sample size. The larger the sample size, the lower the alpha needed such that a significant result can be interpreted as evidence for the alternative. 

The graph below illustrates this relationship for previous example:

```{r}
seqN <- seq(50, 1000, 1)
plot(seqN, alphaN(seqN), type = "l",
     xlab = "n", ylab = "Alpha")
```

## Setting the prior

To set the alpha level as a function of sample size, we need to choose the prior carefully. **alphaN** allows the user to choose from four sensible prior options based on suggestions from the previous literature: Jeffreys' approximate BF (`method = "JAB"`), the minimal training sample (`method = "min"`), the robust minimal training sample (`method = "robust"`), and balanced Type-I and Type-II errors (`method = "balanced"`). `method = "JAB"` is a good choice for users who want to be conservative against small effects, `method = "min"` is for when the MLE is misspecified, `method = "robust"` is for when the MLE is misspecified and the sample size is small, and `method = "balanced"` is for when Type-II errors are costly.  

For instance, to achieve evidence of 3 for 1,000 observations while we ensure balanced error rates, we run

```{r}
alphaN(1000, BF = 3, method = "balanced")
```
The package contains the convenience function `alphaN_plot` that allows a quick comparison of alpha as a function of sample size for the four different methods: 

```{r}
alphaN_plot(BF = 3)
```

---
title: "Intro to alphaN"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Intro to alphaN}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(alphaN)
```

We wish to determine which alpha level is equivalent to a Bayes factor of 1. I.e. only reject the null if the data is at least as likely under the alternative as under the null. To do this, we need a way to connect the $p$-value to the Bayes factor. The **alphaN** package does this for tests of coefficients in regression models.

# Installation

To install the latest release version from CRAN use:

``` r
install.packages("alphaN")
```

You can install the development version of alphaN from [GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("jespernwulff/alphaN")
```

# Basic functionality

This vignette provides an introduction to the basic functionality of **alphaN**. For full details on methodology, please refer to [Wulff & Taylor (2024)](https://doi.org/10.1177/14761270231214429).

## Setting the alpha level

Using the `alphaN` function, we can get the alpha level we need to use to obtain a desired level of evidence when testing a regression coefficient in a regression model.

Here is an example: We are planning to run a linear regression model with 1000 observations. We thus set `n = 1000`. The default `BF` is 1 meaning that we want to avoid Lindley's paradox, i.e. we just want the null and the alternative to be at least equally likely when we reject the null. 

```{r}
alpha <- alphaN(n = 1000, BF = 1)
alpha
```

Therefore, to obtain evidence of at least 1, we should set our alpha to `r round(alpha,4)`.

## Plotting the relationship between the Bayes factor and *p*-value

The `alphaN` function works by mapping the *p*-value to the Bayes factor. This relationship can be shown using the `JAB_plot`. For instance:

```{r}
JAB_plot(n = 1000, BF = 1)
```

The alpha level needed to achieve a Bayes factor of 1 is shown with a red triangle in the plot. Lines for achieving Bayes factors of 3 (moderate evidence) and 10 (strong evidence) are also shown by default. As it is evident a lower alpha level is needed to achieve higher evidence.

## Alpha as a decreasing function of N

An important point of the procedure is that alpha will be set as a function of sample size. The larger the sample size, the lower the alpha needed such that a significant result can be interpreted as evidence for the alternative. 

The graph below illustrates this relationship for the previous example:

```{r}
seqN <- seq(50, 1000, 1)
plot(seqN, alphaN(seqN), type = "l",
     xlab = "n", ylab = "Alpha")
```

## Setting the prior

To set the alpha level as a function of sample size, we need to choose the prior carefully. **alphaN** allows the user to choose from four sensible prior options based on suggestions from the previous literature: Jeffreys' approximate BF (`method = "JAB"`), the minimal training sample (`method = "min"`), the robust minimal training sample (`method = "robust"`), and balanced Type-I and Type-II errors (`method = "balanced"`). `method = "JAB"` is a good choice for users who want to be conservative against small effects, `method = "min"` is for when the MLE is misspecified, `method = "robust"` is for when the MLE is misspecified and the sample size is small, and `method = "balanced"` is for when Type-II errors are costly.  

For instance, to achieve evidence of 3 for 1,000 observations while we ensure balanced error rates, we run

```{r}
alphaN(1000, BF = 3, method = "balanced")
```
The package contains the convenience function `alphaN_plot` that allows a quick comparison of alpha as a function of sample size for the four different methods:

```{r}
alphaN_plot(BF = 3)
```

## Calibrating alpha to effect-size and moment Bayes factors

The four methods above all invert Jeffreys' approximate Bayes factor, whose prior places its mode on a null effect. Klauer, Meyer-Grant & Kellen (2024) propose two alternative families of Bayes factors whose priors instead center the alternative hypothesis on an effect size $d_e$ chosen by the researcher: *effect-size* Bayes factors, and *moment* Bayes factors, under which effects near zero are a priori implausible. `alphaN` can calibrate the alpha level to these Bayes factors, which answers the question: which alpha do I need so that a significant result corresponds to a Bayes factor of at least `BF` against an alternative centered on the effect size I actually care about?

```{r}
# alpha needed at n = 1000 for moderate evidence (BF = 3), targeting a
# medium-sized effect (de = 0.5)
alphaN(1000, BF = 3, method = "ES", de = 0.5)
alphaN(1000, BF = 3, method = "moment", de = 0.5)
```

Because the moment prior rules out effects near zero, the alpha level it implies falls much faster with the sample size than under JAB:

```{r}
ns <- c(100, 1000, 10000)
round(rbind(JAB    = alphaN(ns, BF = 3),
            ES     = alphaN(ns, BF = 3, method = "ES"),
            moment = alphaN(ns, BF = 3, method = "moment")), 5)
```

The prior settings follow the recommendations of Klauer et al. (2024) and can be adjusted through the `nu` and `r` arguments; see `?alphaN` for details, including how to recover the calibration of the default (Jeffreys-Zellner-Siow type) Bayes factor of Rouder et al. (2009).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JAB_plot.R
\name{JAB_plot}
\alias{JAB_plot}
\title{Plots JAB as a function of the p-value}
\usage{
JAB_plot(n, BF = 1, method = "JAB", upper = 1)
}
\arguments{
\item{n}{Sample size. A positive numeric vector.}

\item{BF}{Bayes factor you would like to match. 1 to avoid the Lindley Paradox, 3 to achieve moderate evidence and 10 to achieve strong evidence.}

\item{method}{Used for the choice of 'b'. Currently one of:
\itemize{
\item "JAB": this choice of b produces Jeffreys' approximate BF (Wagenmakers, 2022)
\item "min": uses the minimal training sample for the prior (Gu et al., 2018)
\item "robust": a robust version of "min" that prevents too small b (O'Hagan, 1995)
\item "balanced": this choice of b balances the type I and type II errors (Gu et al., 2016)
}}

\item{upper}{The upper limit for the range of realistic effect sizes. Only relevant when method="balanced". Defaults to 1 such that the range of realistic effect sizes is uniformly distributed between 0 and 1, U(0,1).}
}
\value{
Prints a plot.
}
\description{
Plots JAB as a function of the p-value
}
\examples{
# Plot JAB as function of the p-value for a sample size of 2000
JAB_plot(2000)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alphaN.R
\name{alphaN}
\alias{alphaN}
\title{Set the alpha level based on sample size for coefficients in a regression model}
\usage{
alphaN(n, BF = 1, method = "JAB", upper = 1, de = 0.5, nu = NULL, r = NULL)
}
\arguments{
\item{n}{Sample size. A positive numeric vector.}

\item{BF}{Bayes factor you would like to match. 1 to avoid Lindley's Paradox, 3 to achieve moderate evidence and 10 to achieve strong evidence.}

\item{method}{Which Bayes factor to calibrate alpha to. The first four
options invert Jeffreys' approximate Bayes factor and differ in the choice
of the prior fraction 'b'; the last two invert the exact test-statistic
Bayes factors of Klauer et al. (2024), whose priors center the alternative
hypothesis on a prespecified effect size \code{de}. One of:
\itemize{
\item "JAB": this choice of b produces Jeffreys' approximate BF (Wagenmakers, 2022)
\item "min": uses the minimal training sample for the prior (Gu et al., 2018)
\item "robust": a robust version of "min" that prevents too small b (O'Hagan, 1995)
\item "balanced": this choice of b balances the type I and type II errors (Gu et al., 2016)
\item "ES": calibrates alpha to the effect-size Bayes factor (Klauer et al., 2024)
\item "moment": calibrates alpha to the moment Bayes factor (Klauer et al., 2024), under which effects close to zero are a priori implausible
}}

\item{upper}{The upper limit for the range of realistic effect sizes. Only relevant when method="balanced". Defaults to 1 such that the range of realistic effect sizes is uniformly distributed between 0 and 1, U(0,1).}

\item{de}{The prespecified (targeted) effect size in standardized units
(Cohen's d). Only used by methods "ES" and "moment". Defaults to 0.5, a
medium effect; use 0.2 for small and 0.8 for large effects (Cohen, 1988).}

\item{nu}{Degrees of freedom of the prior t distribution for methods "ES"
and "moment". The default, NULL, uses the values recommended by Klauer
et al. (2024): 3 for "ES" and 5 for "moment".}

\item{r}{Scale of the two prior mixture components for method "ES". The
default, NULL, uses the recommendation of Klauer et al. (2024),
r = sqrt((nu - 2)/nu) * de, which requires nu > 2 and de > 0; otherwise
supply \code{r} explicitly.}
}
\value{
Numeric alpha level required to achieve the desired level of evidence.
}
\description{
Computes the alpha level required to achieve a desired level of evidence,
expressed as a Bayes factor, when testing a coefficient in a regression
model. The alpha level is a decreasing function of the sample size.
Vectorized over \code{n} and \code{BF}.
}
\details{
For methods "ES" and "moment", the alpha level is found by solving for the
critical t value at which the effect-size or moment Bayes factor equals
\code{BF}, and converting that critical value to a two-sided p-value on the t
distribution with \code{n - 1} degrees of freedom (the one-sample /
single-coefficient case of Klauer et al., 2024). The implementation is
validated against the Bayes factors reported in Table 7 of that paper.
Because the moment prior assigns effects near zero a priori density zero,
the alpha level it implies decreases much faster with \code{n} than under JAB.

As a special case, setting \code{method = "ES", nu = 1, de = 0} with an
explicit scale (e.g. \code{r = 1}) calibrates alpha to the default
(Jeffreys-Zellner-Siow type) Bayes factor of Rouder et al. (2009).

For \code{n} greater than 50,000, methods "ES" and "moment" evaluate the
noncentral-t density ratio in its normal limit, which is accurate to a
fraction of a percent there.
}
\section{References}{

Gu et al. (2016). Error probabilities in default Bayesian hypothesis testing. Journal of Mathematical Psychology, 72, 130–143. \cr
\cr
Gu et al. (2018). Approximated adjusted fractional Bayes factors: A general method for testing informative hypotheses. The British Journal of Mathematical and Statistical Psychology, 71(2). \cr
\cr
Klauer, K. C., Meyer-Grant, C. G., & Kellen, D. (2024). On Bayes factors for hypothesis tests. Psychonomic Bulletin & Review. \doi{10.3758/s13423-024-02612-2} \cr
\cr
O’Hagan, A. (1995). Fractional Bayes Factors for Model Comparison. Journal of the Royal Statistical Society. Series B (Methodological), 57(1), 99–138. \cr
\cr
Rouder, J. N., Speckman, P. L., Sun, D., Morey, R. D., & Iverson, G. (2009). Bayesian t tests for accepting and rejecting the null hypothesis. Psychonomic Bulletin & Review, 16, 225–237. \cr
\cr
Wagenmakers, E.-J. (2022). Approximate objective Bayes factors from p-values and sample size: The 3p(sqrt(n)) rule. PsyArXiv. \cr
\cr
Wulff, J. N., & Taylor, L. (2024). How and why alpha should depend on sample size: A Bayesian-frequentist compromise for significance testing. Strategic Organization. \doi{10.1177/14761270231214429}
}

\examples{
# Plot of alpha level as a function of n
seqN <- seq(50, 1000, 1)
plot(seqN, alphaN(seqN), type = "l")

# Alpha calibrated to the effect-size Bayes factor (Klauer et al., 2024),
# targeting moderate evidence for a medium-sized effect
alphaN(1000, BF = 3, method = "ES", de = 0.5)

# The same calibration under the moment Bayes factor
alphaN(1000, BF = 3, method = "moment", de = 0.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JABt.R
\name{JABt}
\alias{JABt}
\title{Transforms a t-statistic into Jeffreys' approximate Bayes factor}
\usage{
JABt(n, t, method = "JAB", upper = 1)
}
\arguments{
\item{n}{Sample size. A positive numeric vector.}

\item{t}{The t-statistic.}

\item{method}{Used for the choice of 'b'. Currently one of:
\itemize{
\item "JAB": this choice of b produces Jeffreys' approximate BF (Wagenmakers, 2022)
\item "min": uses the minimal training sample for the prior (Gu et al., 2018)
\item "robust": a robust version of "min" that prevents too small b (O'Hagan, 1995)
\item "balanced": this choice of b balances the type I and type II errors (Gu et al., 2016)
}}

\item{upper}{The upper limit for the range of realistic effect sizes. Only relevant when method="balanced". Defaults to 1 such that the range of realistic effect sizes is uniformly distributed between 0 and 1, U(0,1).}
}
\value{
A numeric value for the BF in favour of H1.
}
\description{
Converts a t-statistic (or z-statistic) into Jeffreys' approximate Bayes
factor, given the sample size. Vectorized over \code{n} and \code{t}.
}
\examples{
# Transform a t-statistic of 2.695 computed based on a sample size of 200 into JAB
JABt(200, 2.695)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alphaN_plot.R
\name{alphaN_plot}
\alias{alphaN_plot}
\title{Creates a plot of alpha as function of sample size for each of the four prior options}
\usage{
alphaN_plot(BF = 1, max = 10000, ylim = NULL)
}
\arguments{
\item{BF}{Bayes factor you would like to match. 1 to avoid Lindley's Paradox, 3 to achieve moderate evidence and 10 to achieve strong evidence.}

\item{max}{The maximum number of sample size. Defaults to 10,000.}

\item{ylim}{Limits for the y-axis. The default, NULL, covers all four
curves. Set to e.g. c(0, 0.05) to zoom in on small alpha levels.}
}
\value{
Prints a plot.
}
\description{
Creates a plot of alpha as function of sample size for each of the four prior options
}
\examples{
# Plot of alpha level as a function of n for a Bayes factor of 3
alphaN_plot(BF = 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JABp.R
\name{JABp}
\alias{JABp}
\title{Transforms a p-value into Jeffreys' approximate Bayes factor}
\usage{
JABp(n, p, z = TRUE, df = NULL, method = "JAB", upper = 1)
}
\arguments{
\item{n}{Sample size. A positive numeric vector.}

\item{p}{The two-sided p-value.}

\item{z}{Is the p-value based on a z- or t-statistic? TRUE if z.}

\item{df}{If z=FALSE, provide the degrees of freedom for the t-statistic.}

\item{method}{Used for the choice of 'b'. Currently one of:
\itemize{
\item "JAB": this choice of b produces Jeffreys' approximate BF (Wagenmakers, 2022)
\item "min": uses the minimal training sample for the prior (Gu et al., 2018)
\item "robust": a robust version of "min" that prevents too small b (O'Hagan, 1995)
\item "balanced": this choice of b balances the type I and type II errors (Gu et al., 2016)
}}

\item{upper}{The upper limit for the range of realistic effect sizes. Only relevant when method="balanced". Defaults to 1 such that the range of realistic effect sizes is uniformly distributed between 0 and 1, U(0,1).}
}
\value{
A numeric value for the BF in favour of H1.
}
\description{
Converts a two-sided p-value from a z- or t-test into Jeffreys' approximate
Bayes factor, given the sample size.
}
\examples{
# Transform a p-value of 0.007038863 from a z-test into JAB
# using a sample size of 200.
JABp(200, 0.007038863)

# Transform a p-value of 0.007038863 from a t-test with 190
# degrees of freedom into JAB using a sample size of 200.
JABp(200, 0.007038863, z=FALSE, df=190)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JAB.R
\name{JAB}
\alias{JAB}
\title{Transforms a t-statistic from a glm or lm object into Jeffreys' approximate Bayes factor}
\usage{
JAB(glm_obj, covariate, method = "JAB", upper = 1)
}
\arguments{
\item{glm_obj}{a glm or lm object.}

\item{covariate}{the name of the covariate that you want a BF for as a string.}

\item{method}{Used for the choice of 'b'. Currently one of:
\itemize{
\item "JAB": this choice of b produces Jeffreys' approximate BF (Wagenmakers, 2022)
\item "min": uses the minimal training sample for the prior (Gu et al., 2018)
\item "robust": a robust version of "min" that prevents too small b (O'Hagan, 1995)
\item "balanced": this choice of b balances the type I and type II errors (Gu et al., 2016)
}}

\item{upper}{The upper limit for the range of realistic effect sizes. Only relevant when method="balanced". Defaults to 1 such that the range of realistic effect sizes is uniformly distributed between 0 and 1, U(0,1).}
}
\value{
A numeric value for the BF in favour of H1.
}
\description{
Extracts the test statistic of a coefficient from a fitted model object and
converts it into Jeffreys' approximate Bayes factor, given the sample size
used in the fit.
}
\examples{
# Simulate data

## Sample size
n <- 200

## Regressors
Z1 <- runif(n, -1, 1)
Z2 <- runif(n, -1, 1)
Z3 <- runif(n, -1, 1)
Z4 <- runif(n, -1, 1)
X <- runif(n, -1, 1)

## Error term
U <- rnorm(n, 0, 0.5)

## Outcome
Y <- X/sqrt(n) + U

# Run a GLM
LM <- glm(Y ~ X + Z1 + Z2 + Z3 + Z4)

# Compute JAB for "X" based on the regression results
JAB(LM, "X")

# Compute JAB using the minimum prior
JAB(LM, "X", method = "min")
}
